@@ -1,14 +1,14 @@
 import os
 
-import numpy as np
-
+import astropy.constants as const
 import astropy.coordinates as coord
 import astropy.time
-from astropy import units as u
-
+import numpy as np
 import sunpy.io
 import sunpy.map
 import sunpy.time
+from astropy import units as u
+from astropy.wcs import WCS
 
 import pfsspy.map
 
@@ -47,11 +47,13 @@ def fix_hmi_meta(hmi_map):
         hmi_map.meta['cdelt1'] = np.abs(hmi_map.meta['cdelt1'])
 
     if 'date-obs' not in hmi_map.meta and 't_obs' in hmi_map.meta:
-        hmi_map.meta['date-obs'] = sunpy.time.parse_time(hmi_map.meta['t_obs']).isot
+        hmi_map.meta['date-obs'] = sunpy.time.parse_time(
+            hmi_map.meta['t_obs']).isot
 
     # Fix observer coordinate
     if 'hglt_obs' not in hmi_map.meta:
-        hmi_map.meta.update(pfsspy.map._earth_obs_coord_meta(hmi_map.meta['date-obs']))
+        hmi_map.meta.update(pfsspy.map._earth_obs_coord_meta(
+            hmi_map.meta['date-obs']))
 
 
 def load_adapt(adapt_path):
@@ -109,18 +111,18 @@ def carr_cea_wcs_header(dtime, shape):
     obstime = dtime or astropy.time.Time('2000-1-1')
 
     frame_out = coord.SkyCoord(
-        0 * u.deg, 0 * u.deg, obstime=obstime,
-        frame="heliographic_carrington", observer='sun')
+        0 * u.deg, 0 * u.deg, const.R_sun, obstime=obstime,
+        frame="heliographic_carrington", observer='self')
     # Construct header
     header = sunpy.map.make_fitswcs_header(
-        shape, frame_out,
+        [shape[1], shape[0]], frame_out,
         scale=[360 / shape[0],
                180 / shape[1]] * u.deg / u.pix,
         reference_pixel=[(shape[0] / 2) - 0.5,
                          (shape[1] / 2) - 0.5] * u.pix,
         projection_code="CEA")
 
-    # Fix CELT for lat axis
+    # Fix CDELT for lat axis
     header['CDELT2'] = (180 / np.pi) * (2 / shape[1])
     # pop out the time if it isn't supplied
     if dtime is None:
@@ -193,18 +195,18 @@ def is_full_sun_synoptic_map(m, error=False):
 def _is_full_sun_car(m, error=False):
     shape = m.data.shape
 
-    dphi = m.meta['cdelt1']
-    phi = shape[1] * dphi
-    if not np.allclose(phi, 360, atol=0.1):
+    dphi = m.scale.axis1
+    phi = shape[1] * u.pix * dphi
+    if not np.allclose(np.abs(phi), 360 * u.deg, atol=0.1 * u.deg):
         if error:
             raise ValueError('Number of points in phi direction times '
                              'CDELT1 must be close to 360 degrees. '
                              f'Instead got {dphi} x {shape[0]} = {phi}')
         return False
 
-    dtheta = m.meta['cdelt2']
-    theta = shape[0] * dtheta
-    if not np.allclose(theta, 180, atol=0.1):
+    dtheta = m.scale.axis2
+    theta = shape[0] * u.pix * dtheta
+    if not np.allclose(theta, 180 * u.deg, atol=0.1 * u.deg):
         if error:
             raise ValueError('Number of points in theta direction times '
                              'CDELT2 must be close to 180 degrees. '
@@ -216,23 +218,24 @@ def _is_full_sun_car(m, error=False):
 def _is_full_sun_cea(m, error=False):
     shape = m.data.shape
 
-    dphi = m.meta['cdelt1']
-    phi = shape[1] * dphi
-    if not np.allclose(phi, 360, atol=0.1):
+    dphi = m.scale.axis1
+    phi = shape[1] * u.pix * dphi
+    if not np.allclose(np.abs(phi), 360 * u.deg, atol=0.1 * u.deg):
         if error:
             raise ValueError('Number of points in phi direction times '
                              'CDELT1 must be close to 360 degrees. '
                              f'Instead got {dphi} x {shape[1]} = {phi}')
         return False
 
-    dtheta = m.meta['cdelt2']
-    theta = shape[0] * dtheta * np.pi / 2
-    if not np.allclose(theta, 180, atol=0.1):
+    dtheta = m.scale.axis2
+    theta = shape[0] * u.pix * dtheta * np.pi / 2
+    if not np.allclose(theta, 180 * u.deg, atol=0.1 * u.deg):
         if error:
             raise ValueError('Number of points in theta direction times '
                              'CDELT2 times pi/2 must be close to '
                              '180 degrees. '
-                             f'Instead got {dtheta} x {shape[0]} * pi/2 = {theta}')
+                             f'Instead got {dtheta} x {shape[0]} * pi/2 '
+                             f'= {theta}')
         return False
     return True
 
@@ -266,8 +269,8 @@ def car_to_cea(m, method='interp'):
     --------
     :mod:`reproject` for the methods that perform the reprojection.
     """
-    from astropy.wcs import WCS
-    from reproject import reproject_interp, reproject_exact, reproject_adaptive
+    # Add reproject import here to avoid import dependency
+    from reproject import reproject_adaptive, reproject_exact, reproject_interp
     methods = {'adaptive': reproject_adaptive,
                'interp': reproject_interp,
                'exact': reproject_exact}
@@ -279,19 +282,83 @@ def car_to_cea(m, method='interp'):
     is_full_sun_synoptic_map(m, error=True)
     is_car_map(m, error=True)
 
+    # Create output FITS header
     header_out = m.wcs.to_header()
     header_out['CTYPE1'] = header_out['CTYPE1'][:5] + 'CEA'
     header_out['CTYPE2'] = header_out['CTYPE2'][:5] + 'CEA'
     header_out['CDELT2'] = 180 / np.pi * 2 / m.data.shape[0]
     wcs_out = WCS(header_out, fix=False)
-    # Check if we need to add the heliographic observer (in sunpy <2.1)
-    if hasattr(m.wcs, 'heliographic_observer'):
-        wcs_out.heliographic_observer = m.observer_coordinate
+
+    # Reproject data
     data_out = reproject(m, wcs_out, shape_out=m.data.shape,
                          return_footprint=False)
 
-    meta_out = m.meta.copy()
-    for key in ['CTYPE1', 'CTYPE2', 'CDELT2']:
-        meta_out[key] = header_out[key]
-    m_out = sunpy.map.Map(data_out, header_out)
-    return m_out
+    return sunpy.map.Map(data_out, header_out)
+
+
+@u.quantity_input
+def roll_map(m, lh_edge_lon: u.deg = 0.0 * u.deg, method='interp'):
+    """
+    Roll an input synoptic map so that it's left edge corresponds to a specific
+    Carrington longitude.
+
+    Roll is performed by changing the FITS header parameter "CRVAL1"
+    to the new value of the reference pixel (FITS header parameter CRPIX1)
+    corresponding to aligning the left hand edge of the map with
+    lh_edge_lon. The altered header is provided to reproject to produce
+    the new map.
+
+    Parameters
+    ----------
+    m : sunpy.map.GenericMap
+        Input map
+    lh_edge_lon : float
+        Desired Carrington longitude (degrees) for left hand edge of map.
+        Default is 0.0 which results in a map with the edges at 0/360 degrees
+        Carrington  longitude. Input value must be in the range [0,360]
+    method : str
+        Reprojection method to use. Can be ``'interp'`` (default),
+        ``'exact'``, or ``'adaptive'``. See :mod:`reproject` for a
+        description of the different methods. Note that different methods will
+        give different results, and not all will conserve flux.
+
+    Returns
+    -------
+    output_map : sunpy.map.GenericMap
+        Re-projected map. All metadata is preserved, apart from CRVAL1  which
+        encodes the longitude of the reference pixel in the image, and which
+        is updated to produce the correct roll.
+
+    See also
+    --------
+    :mod:`reproject` for the methods that perform the reprojection.
+    """
+    # Add reproject import here to avoid import dependency
+    from reproject import reproject_adaptive, reproject_exact, reproject_interp
+    methods = {'adaptive': reproject_adaptive,
+               'interp': reproject_interp,
+               'exact': reproject_exact}
+    if method not in methods:
+        raise ValueError(f'method must be one of {methods.keys()} '
+                         f'(got {method})')
+    if lh_edge_lon > 360.0 * u.deg or lh_edge_lon < 0.0 * u.deg:
+        raise ValueError(f"lh_edge_lon must be in the range [0,360])")
+
+    reproject = methods[method]
+    # Check input map is valid
+    is_full_sun_synoptic_map(m, error=True)
+
+    # Create output FITS header
+    header_out = m.wcs.to_header()
+    # Note half pixel shift to ensure LH edge leftmost pixel is
+    # correctly aligned with map LH edge
+    header_out['CRVAL1'] = (lh_edge_lon - 0.5 * header_out['CDELT1'] * u.deg +
+                            header_out['CRPIX1'] * header_out['CDELT1'] *
+                            u.deg).to_value(u.deg) % 360
+    wcs_out = WCS(header_out, fix=False)
+
+    # Reproject data
+    data_out = reproject(m, wcs_out, shape_out=m.data.shape,
+                         return_footprint=False)
+
+    return sunpy.map.Map(data_out, header_out)
